from datetime import timedelta
from django.contrib.auth import authenticate, get_user_model
from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.permissions import AllowAny
from rest_framework_simplejwt.tokens import RefreshToken

User = get_user_model()  # Get custom user model

class VerifySuperuserView(APIView):
    permission_classes = [AllowAny]  # Allows unauthenticated access to this view

    def post(self, request):
        email = request.data.get('email')
        password = request.data.get('password')

        print(email)
        print(password)

        # Try to authenticate using Django's built-in authentication system
        user = authenticate(request, email=email, password=password)
        print(user, 'user')

        # If authentication fails, check manually (useful for custom User models)
        if user is None:
            user = User.objects.filter(email=email).first()
            print(user, 'aat alaa')
            if user and user.check_password(password):  # Manually verify password
                print("DEBUG: Password check passed")  # Debugging
            else:
                print(f"DEBUG: Authentication failed for {email}")  # Debugging
                return Response({
                    'success': False,
                    'message': 'Invalid credentials'
                }, status=400)

    
        refresh = RefreshToken.for_user(user)
        
        access_token = str(refresh.access_token)  
        refresh_token = str(refresh)
        print(access_token,'accesss_token')
        print(refresh_token,'refresh toke')
        return Response({
            'success': True,
            'code': 200,
            'is_superuser': user.is_superuser,
            'message': 'User is superuser' if user.is_superuser else 'User is not superuser',
            'access_token': access_token, 
            'refresh_token': refresh_token  
        })

  

from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework import status
from contactus.models import Country, City ,ContactUs
from .serializers import CountrySerializer, CitySerializer ,ContactusSerializer
from gallery.utils import paginate_queryset_and_serialize


class CountryAPIView(APIView):

    def get(self, request, *args, **kwargs):
        if 'pk' in kwargs:
            # Retrieve a single Country item by id
            try:
                country = Country.objects.get(id=kwargs['pk'])
                serializer = CountrySerializer(country)
                return Response(serializer.data, status=status.HTTP_200_OK)
            except Country.DoesNotExist:
                return Response({"detail": "Not found."}, status=status.HTTP_404_NOT_FOUND)
        else:
            search_term = request.query_params.get('term', None)

            # Handle "undefined" or empty search term
            if not search_term or search_term == "undefined":
                search_term = None

            # Filter the Country items based on the search term
            countries = Country.objects.filter(name__icontains=search_term,deleted_at__isnull=True) if search_term else Country.objects.filter(deleted_at__isnull=True)

            # Ensure proper ordering by `id` in descending order
            countries = countries.order_by('-id')

            # Paginate and serialize the queryset
            paginated_response = paginate_queryset_and_serialize(countries, request, CountrySerializer, search_term=search_term)

            # Return the paginated response
            return paginated_response
        

    def post(self, request, *args, **kwargs):
        # Handle Add (Create)
        serializer = CountrySerializer(data=request.data)

        if serializer.is_valid():
            serializer.save(status=True)  # Set the status to True before saving
            return Response({
                "success": True,
                "code": 201,
                "message": "Country Saved Successfully.",
                "data": serializer.data
            }, status=status.HTTP_201_CREATED)
        
        return Response({
            "success": False,
            "code": 400,
            "message": "Failed to create Country",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def put(self, request, *args, **kwargs):
        try:
            # Fetch the existing Country object
            country = Country.objects.get(id=kwargs['pk'])
            original_status = country.status  # Save the current status
        except Country.DoesNotExist:
            return Response({"detail": "Not found."}, status=status.HTTP_404_NOT_FOUND)

        # Use the serializer to validate and update the Country object
        serializer = CountrySerializer(country, data=request.data, partial=True)

        if serializer.is_valid():
            # Ensure the status field remains unchanged if it's not explicitly provided
            if 'status' not in request.data:
                country.status = original_status  # Restore the original status

            serializer.save()
            return Response({
                "success": True,
                "code": 200,
                "message": "Country updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        return Response({
            "success": False,
            "code": 400,
            "message": "Failed to update Country.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def delete(self, request, *args, **kwargs):
        # Handle Delete
        try:
            country = Country.objects.get(id=kwargs['pk'])
            country.delete()
        except Country.DoesNotExist:
            return Response({
                "success": False,
                "code": 404,
                "message": "Country not found.",
            }, status=status.HTTP_404_NOT_FOUND)

        return Response({
            "success": True,
            "code": 200,
            "message": "Country deleted successfully."
        }, status=status.HTTP_200_OK)

    def patch(self, request, *args, **kwargs):
        # Handle Partial Update (Toggle `status`)
        try:
            country = Country.objects.get(id=kwargs['pk'])
        except Country.DoesNotExist:
            return Response({"detail": "Not found."}, status=status.HTTP_404_NOT_FOUND)

        # Toggle the `status` of the country
        country.status = not country.status
        country.save()

        # Serialize and return the updated country
        serializer = CountrySerializer(country)
        return Response({
                "success": True,
                "code": 200,
                "message": "Country updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)


class CountryListWithoutPaginationView(APIView):
    def get(self, request, *args, **kwargs):
        # Fetch all Country objects
        countries = Country.objects.all()

        # Serialize the data
        serializer = CountrySerializer(countries, many=True)

        # Return the response in the desired format
        return Response({
            "results": serializer.data,
            "count": len(serializer.data),  # Count of all records
            "success": True,
            "message": "All countries fetched successfully"
        }, status=status.HTTP_200_OK)

    
class CityListWithoutPaginationView(APIView):
    def get(self, request, *args, **kwargs):
        # Fetch all City objects
        countries = City.objects.filter(deleted_at__isnull=True)

        # Serialize the data
        serializer = CitySerializer(countries, many=True)

        # Return the response in the desired format
        return Response({
            "results": serializer.data,
            "count": len(serializer.data),  # Count of all records
            "success": True,
            "message": "All countries fetched successfully"
        }, status=status.HTTP_200_OK)

class CityAPIView(APIView):

    def get(self, request, *args, **kwargs):
        if 'pk' in kwargs:
            # Retrieve a single City item by id
            try:
                city = City.objects.get(id=kwargs['pk'])
                serializer = CitySerializer(city)
                return Response(serializer.data, status=status.HTTP_200_OK)
            except City.DoesNotExist:
                return Response({"detail": "Not found."}, status=status.HTTP_404_NOT_FOUND)
        else:
            search_term = request.query_params.get('term', None)

            # Handle "undefined" or empty search term
            if not search_term or search_term == "undefined":
                search_term = None

            # Filter the City items based on the search term
            cities = City.objects.filter(name__icontains=search_term,deleted_at__isnull=True) if search_term else City.objects.filter(deleted_at__isnull=True)

            # Ensure proper ordering by `id` in descending order
            cities = cities.order_by('-id')

            # Paginate and serialize the queryset
            paginated_response = paginate_queryset_and_serialize(cities, request, CitySerializer, search_term=search_term)

            # Return the paginated response
            return paginated_response

    def post(self, request, *args, **kwargs):
        # Handle Add (Create)
        serializer = CitySerializer(data=request.data)

        if serializer.is_valid():
            serializer.save(status=True)  # Set the status to True before saving
            return Response({
                "success": True,
                "code": 201,
                "message": "City Saved Successfully.",
                "data": serializer.data
            }, status=status.HTTP_201_CREATED)
        
        return Response({
            "success": False,
            "code": 400,
            "message": "Failed to create City",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def put(self, request, *args, **kwargs):
        try:
            # Fetch the existing City object
            city = City.objects.get(id=kwargs['pk'])
            original_status = city.status  # Save the current status
        except City.DoesNotExist:
            return Response({"detail": "Not found."}, status=status.HTTP_404_NOT_FOUND)

        # Use the serializer to validate and update the City object
        serializer = CitySerializer(city, data=request.data, partial=True)

        if serializer.is_valid():
            # Ensure the status field remains unchanged if it's not explicitly provided
            if 'status' not in request.data:
                city.status = original_status  # Restore the original status

            serializer.save()
            return Response({
                "success": True,
                "code": 200,
                "message": "City updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

        return Response({
            "success": False,
            "code": 400,
            "message": "Failed to update City.",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)

    def delete(self, request, *args, **kwargs):
        # Handle Delete
        try:
            city = City.objects.get(id=kwargs['pk'])
            city.delete()
        except City.DoesNotExist:
            return Response({
                "success": False,
                "code": 404,
                "message": "City not found.",
            }, status=status.HTTP_404_NOT_FOUND)

        return Response({
            "success": True,
            "code": 200,
            "message": "City deleted successfully."
        }, status=status.HTTP_200_OK)

    def patch(self, request, *args, **kwargs):
        # Handle Partial Update (Toggle `status`)
        try:
            city = City.objects.get(id=kwargs['pk'])
        except City.DoesNotExist:
            return Response({"detail": "Not found."}, status=status.HTTP_404_NOT_FOUND)

        # Toggle the `status` of the city
        city.status = not city.status
        city.save()

        # Serialize and return the updated city
        serializer = CitySerializer(city)
        return Response({
                "success": True,
                "code": 200,
                "message": "City updated successfully.",
                "data": serializer.data
            }, status=status.HTTP_200_OK)

class ContactusSave(APIView):
    def post(self, request, *args, **kwargs):
        # Handle Add (Create)
        serializer = ContactusSerializer(data=request.data)
        print(request.data,'aaaaa')
        
        if serializer.is_valid():
            print(serializer.is_valid(),'aaaaasss')
            serializer.save()  # Set the status to True before saving
            return Response({
                "success": True,
                "code": 201,
                "message": "Record Saved Successfully.",
                "data": serializer.data
            }, status=status.HTTP_201_CREATED)
        
        return Response({
            "success": False,
            "code": 400,
            "message": "Failed to create Record",
            "errors": serializer.errors
        }, status=status.HTTP_400_BAD_REQUEST)
    
    def get(self, request, *args, **kwargs):
       
        if 'pk' in kwargs:
            # Retrieve a single Country item by id
            try:
                
                country = ContactUs.objects.get(id=kwargs['pk'])
                serializer = ContactusSerializer(country)
                return Response(serializer.data, status=status.HTTP_200_OK)
            except ContactUs.DoesNotExist:
                return Response({"detail": "Not found."}, status=status.HTTP_404_NOT_FOUND)
        else:
            search_term = request.query_params.get('term', None)

            # Handle "undefined" or empty search term
            if search_term:
                countries = ContactUs.objects.filter(full_name__icontains=search_term, deleted_at__isnull=True)
            else:
                countries = ContactUs.objects.filter(deleted_at__isnull=True)

            # Ensure proper ordering by `id` in descending order
            countries = countries.order_by('-id')
           
            # Paginate and serialize the queryset
            paginated_response = paginate_queryset_and_serialize(countries, request, ContactusSerializer, search_term=search_term)

            # Return the paginated response
            return paginated_response