from django.db import models
from gallery.utils import SoftDeleteModel, SoftDeleteManager

# Create your models here.

class Country(SoftDeleteModel):
    
    name = models.CharField(max_length=500, null=True, blank=True)
    status = models.BooleanField(default=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)  # Note: corrected to auto_now for update timestamp
    deleted_at = models.DateTimeField(null=True, blank=True)

class City(SoftDeleteModel):
    country = models.ForeignKey(Country, on_delete=models.CASCADE,
                                     related_name="country_id",null=True, blank=True)
    name = models.CharField(max_length=500, null=True, blank=True)
    status = models.BooleanField(default=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)  # Note: corrected to auto_now for update timestamp
    deleted_at = models.DateTimeField(null=True, blank=True)


class Product(SoftDeleteModel):
    
    name = models.CharField(max_length=500, null=True, blank=True)
    status = models.BooleanField(default=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)  # Note: corrected to auto_now for update timestamp
    deleted_at = models.DateTimeField(null=True, blank=True)

class ContactUs(SoftDeleteModel):
    
    product   = models.ForeignKey(Product, on_delete=models.CASCADE,
                                     related_name="product",null=True, blank=True)
    full_name = models.CharField(max_length=500, null=True, blank=True)
    email = models.CharField(max_length=100, null=True, blank=True)
    country_code = models.CharField(max_length=10, null=True, blank=True ,default=+91)
    mobile_no = models.CharField(max_length=20, null=True, blank=True)
    # country = models.ForeignKey(Country, on_delete=models.CASCADE,
    #                                  related_name="country")
    # city = models.ForeignKey(City, on_delete=models.CASCADE,
    #                                  related_name="city")
    country = models.CharField(max_length=500, null=True, blank=True)
    city = models.CharField(max_length=500, null=True, blank=True)
    message = models.TextField(null=True, blank=True)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)  # Note: corrected to auto_now for update timestamp
    objects = SoftDeleteManager()  # Use custom manager to filter out deleted objects

    # Explicitly declare the 'deleted_at' field here to control its order
    deleted_at = models.DateTimeField(null=True, blank=True)

    def __str__(self):
        return self.full_name
